<?php
// $Id: NoAhContentList.php,v 1.4 2005/03/22 01:57:22 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhContentList.php provides methods for handling a NoAh defined Content List
 *
 * A NoAh content list is an rdf-xml file that defines a 'list' of information, such
 * as, a list of categories, a list of countries, ect. All list are made up of a 
 * series of 'label','value','description' combinations each representing a single
 * row or item in a list.
 * 
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */
 
/**
 * Require the NoAhRap class. NOTE: could probably remove this.
 */
require_once(CORE_PATH.'/class/noahlib/NoAhRap.php');

/**
 * NoAhContentList extends NoAh rap and will fetch and return values and items from
 * a NoAh defined content list.
 * @package noahlib
 */
class NoAhContentList extends NoAhRap
{
    /**
     * Name of the Content List
     * @var     string   
     */
   var $name;
   /**
    * Name of the XML file with content list details
    * @var     string   
    */
   var $filepath;
   /**
    * A label or user friendly version of {@link $name}
    * @var     string   
    */
   var $label;   
   /**
    * A description of our content list
    * @var     string   
    */
   var $desc;   
   /**
    * An array of data parsed from {@link $filepath}
    * @var     array   
    */
   var $xdata = array();
   /**
    * An array of list values
    * @var     string   
    */
   var $values = array();
   /**
    * An array of list details as pulled from {@link $filepath}
    * @var     string   
    */
   var $xlist = array();
   /**
    * An array of list values as pulled from {@link $filepath}
    * @var     string   
    */
   var $xvalues = array();
   /**
    * An array which can temporarily hold a new value before list has been saved.
    * @var     string   
    */
   var $newvalue = array();
   
   /**
    * Class constructor method 
    * 
    * Validates the name of a content list and if valid reads the xml file
    * Sets up {@link $name} {@link filepath}
    * @see readFile()
    * @see normalizeName()
    * @param string $name name of the content list we want to load
    * @return void
    */
    function NoAhContentList( $name )
    {
        $this->name = $this->normalizeName( $name );
        
        // Check to make sure we have the file
        // Check to see if we can find a file for this source.
        if ( is_file(CORE_PATH.'/xml/'.$this->name ) ) {
            $this->filepath = CORE_PATH.'/xml/'.$this->name;
        }else if(is_file(XML_PATH.'/'.$this->name )){
            $this->filepath = XML_PATH.'/'.$this->name;
        }
        
        if ($this->filepath){
           $this->readFile();
        } 
    }

    /**
     * Adds a new {@link $values} set to the content list
     * @param string $label the label to use for our value set
     * @param string $value the value of our value set
     * @param string $desc a description of our value
     * @param string $order the order our value should appear in our list
     * @return void
     */
    function addValue( $label, $value, $desc='', $order=55 )
    {
       if ( $label==='' || $value==='' ) return false;

         $set = array ( 
         	'label' => $label,
         	'value' => $value,
         	'desc' => $desc,
         	'order' => $order
         );
         $this->values[] = $set;
    }
    
    /**
     * Returns an array of details about our list
     * @return array
     */
    function getListDetails() {
        $det['name'] = $this->name;
        if ( ! $this->label ) $this->label = $this->name;
        $det['label'] = $this->label;
        $det['desc'] = $this->desc;
        $det['newvalue'] = $this->newvalue;
      return $det;  
    }
    
    /**
     * Returns all {@link $values} in our list.
     * @return array
     */
    function getListValues() {
        return $this->sortListValues();
    }

    function sortListValues()
    {
        $arr1 = array();
        foreach( $this->values as $i=>$row )
        {
            $arr1[$i] =  array_map('strtolower', $row);
        }
        $values = array_csort( $arr1,'order',SORT_ASC,'label',SORT_ASC );

        foreach( $values as $i=>$lower )
        {
            foreach( $this->values as $ii=>$upper )
            {
                if ( $lower['label'] == strtolower($upper['label']) ) {
                    //error_log($lower['label']." ======= ".$upper['label']);
                    $values[$i] = $upper;
                }
            }
        }
        return $values;
    }

    /**
     * A comparison method that can be used with php usort() function and is called in {@link getListValues}
     * @return array
     */ 
    
    /**
     * Converts a name to a full XML file name by adding .xml extension and list_ prefix
     * 
     * Returns the converted name. 
     * 
     * @param string $name we want to normalize.
     * @return integer
     */
    function normalizeName($name)
    {
        if ( substr(strtolower($name),-4)!='.xml') {
            $name = $name.'.xml';
        }
        if ( strpos($name,'list_')===false ) {
            $name = 'list_'.$name;
        }
        
        $name = str_replace(' ','',$name);
        
        return strtolower($name);
    }
    
    /**
     * Reads the contents of a Content List XML file
     * 
     * Method sets up {@link $xlist} {@link $xvalues} {@link xdata}
     * and will also loop over {@link $xvalues} and add each found
     * value to {@link $values}
     * 
     * @see addValue()
     * @return void
     */
    function readFile()
    {
	  $this->NoAhRap( $this->filepath );
	  $this->xdata = $this->getData();

	  // Find out list and values in our xml data
	  foreach($this->xdata as $uri => $data ) {
            // look for list namespace.
            if ( substr($uri,0,5)=='list/') {
                $this->xlist = $data;
            }
            if ( substr($uri,0,9)=='listitem/') {
                $this->xvalues[] = $data;
            }
	   }
	   
	   if ( isset($this->xlist['name']) ) { 
	     $this->name = $this->xlist['name'];
	   }
	   if ( isset($this->xlist['label']) ) { 
	      $this->label = $this->xlist['label'];
	   }
	   
	   foreach ( $this->xvalues as $i=>$value ) {
	       $label = (isset($value['label'])) ? $value['label'] : '' ;
           $thisvalue = (isset($value['value'])) ? $value['value'] : '' ;
           $desc = (isset($value['desc'])) ? $value['desc'] : '' ;
           $order = (isset($value['order'])) ? $value['order'] : 55 ;
           $this->addValue( $label, $thisvalue, $desc, $order );
	   }
	   
       
    }
    
    /**
     * Loads list values from a $_SESSION variable.
     * 
     * This method is specific to the functionality of NoAh Admin tools.
     * @see addValue()
     * @return void
     */
    function loadFromMemory()
    {
    	if ( isset($_SESSION['mem']['list']) && $_SESSION['mem']['list'] ){
    	   foreach($_SESSION['mem']['list'] as $k=>$v)
    	   {
    	   		$this->$k = $v;
    	   }
    	}
    	if ( isset($_SESSION['mem']['values']) && $_SESSION['mem']['values'] ){
    		$this->values = array();
    	   foreach($_SESSION['mem']['values'] as $k=>$value){
		       $label = (isset($value['label'])) ? $value['label'] : '' ;
	           $thisvalue = (isset($value['value'])) ? $value['value'] : '' ;
	           $desc = (isset($value['desc'])) ? $value['desc'] : '' ;
   	    	   $order = (isset($value['order'])) ? $value['order'] : 55 ;
   		       $this->addValue( $label, $thisvalue, $desc, $order );
    	   }
    	}
    
    }
    
    /**
     * Changes a {@link $values} record based at a specific index.
     * @param string $item the label/name of the value set
     * @param string $value the value of the value set     
     * @param integer $index the index key of the value to change     
     * @return void
     */
    function changeValue( $item, $value, $index )
    {	
    	error_log("change $item to $value at $index");
    	if ( $index == 'new' ) {
    		$this->newvalue[$item] = $value;
    		return;
    	}
    	$this->values[$index][$item] = $value;
    }

    // Checks to see if a value already exsists
    function isValue( $value )
    {
        foreach ( $this->values as $i=>$varr )
        {
            if ( $varr['value'] == $value ) {
                return $i;
            }
        }
       return false;
    }
    
    /**
     * Deletes an array of indexes from {@link $values}, and reindex remaining values.
     * @param array $indexes an array of indexes to delete
     * @return void
     */
    function deleteValues( $indexes )
    {
    	
    	foreach($indexes as $i=>$id )
    	{
    		error_log($id);
    		if ( $id === '' ) continue;
    		unset($this->values[$id]);
    	}
    	
    	// Reindex our values
    	$myvalues = $this->values;
    	$this->values = array();
    	foreach($myvalues as $i=>$value )
    	{
    		$this->values[] = $value;
    	}
    	return;
    }
    
    
    
    /**
     * Saves a Content List XML file and returns the results of that effort.
     * @see NoAhRdfModel()
     * @return string
     */
    function saveListXML()
    {
    
    $list = $this->getListDetails();
    $values = $this->getListValues();
    
	// Include our rdfmodeler class file.
	require_once( CORE_PATH.'/modules/nosystem/class/rdfmodel.php' );


	/**
	 * Start save source process.
	 */

	$mymodel = new NoAhRdfModel();

	// DEFINE STATEMENTS FOR LIST HEADER INFROMATION

	// Set our Source URL
	$luri = '../ns/list/' . str_replace( ' ','',$list['label']);
	$mymodel->addStatement($luri,'RDF:type','RDF:Seq');

	// label
	$mymodel->addStatement($luri,'noah:label',$list['label']);
	// name
	$mymodel->addStatement($luri,'noah:name',$list['name']);
	// table


	// DEFINE STATEMENTS FOR EACH CONTENT PIECE
	foreach($values as $i =>$myvalue )
	{   
   
	   // Set the url for our piece.
	   $uri = '../ns/listitem/'. str_replace(' ','', $myvalue['label']);
	   
	   // Add to our pieces sequence
	   $mymodel->addStatement($luri,'RDF:li',$uri);
   
	   // Add statements specific for this piece.
	   $mymodel->addStatement($uri,'noah:label',$myvalue['label']);
   		$mymodel->addStatement($uri,'noah:value',$myvalue['value']);
	   if (isset($myvalue['desc'])) $mymodel->addStatement($uri,'noah:desc',$myvalue['desc']);
	   if (isset($myvalue['order'])) $mymodel->addStatement($uri,'noah:order',$myvalue['order']);
   
	   // kill our URI to prevent accidentally using it again
	   unset($uri);

	}

	// SET DEFAULT STATEMENTS WHICH NEED TO BE INCLUDED IN ALL SOURCES

	// Set the name of our model
	$mymodel->setName( $this->normalizeName($list['name']) );

	// Save our model
	if ( ! $mymodel->saveModel() ){
	     $results = "Failed to Save Content Source File.";
	} else {
	     $results = "success";
	}
    
    return $results;
    
    }
    
}

if ( ! function_exists('orderComparison') ) {
    function orderComparison($a,$b){
      if ($a['order'] == $b['order']) {
           return 0;
       }
       return ($a['order'] < $b['order']) ? -1 : 1;
    }
}

if ( ! function_exists('array_csort') ) {
function array_csort() {  
 $args = func_get_args();
 $marray = array_shift($args);
 $msortline = "return(array_multisort(";
 $i=0;
 foreach ($args as $arg) {
  $i++;
  if (is_string($arg)) {
   foreach ($marray as $row) {
   $sortarr[$i][] = $row[$arg];
   }
  } else {
  $sortarr[$i] = $arg;
  }
  $msortline .= "\$sortarr[".$i."],";
 }
 $msortline .= "\$marray));";

 @eval($msortline);
 return $marray;
}
 
}


?>
