<?php
// $Id: NoAhContentReader.php,v 1.2 2005/03/22 01:57:22 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhContentReader.php provides methods for getting content from a NoAh
 * Content Source.
 *
 * NoAh provides the ability to define metadata about Content Sources, this class
 * lets you define parameters about how to fetch content and then executes the
 * retrieval of content.
 * 
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */

/**
 * Make sure we have included NoAhContentSource class
 */
require_once( CORE_PATH.'/class/noahlib/NoAhContentSource.php');

/**
 * NoAhContentReader provides methods for getting content from a NoAh Content Source.
 * 
 * This class is used in place of of normal ContentSources object specifically
 * when content needs to be retrieved.
 * 
 * @package noahlib
 */
class NoAhContentReader extends NoAhContentSource
{

    /**
     * An array of content rows 
     * @var     array
     */
    var $rows = array();

    /**
     * An array of content columns
     * @var     array
     */
    var $columns = array();

    /**
     * The total number of content rows retrieved
     * @var 	integer
     */
     var $row_count;
     /**
      * The name of the content piece to sort content by
      * @var 	string
      */
     var $sortby;
     /**
      * The direction to sort content. 'ASC' or 'DESC'
      * @var 	string
      */
     var $sortdir;
     /**
      * The page number to display when reading paged records
      * @var 	string
      */
     var $page;
     /**
      * The number of content records to retrieve
      * @var 	string
      */
     var $limit;
     /**
      * An array of content retrieved from the content source
      * @var 	array
      */
     var $content = array();
     /**
      * An array of custom properties which can be passed to {@link $fetch}
      * @var 	array
      */
     var $custom_prop = array();
     
    /**
     * An array of filter properties which can be used to search or filter content
     * @var     array
     */
     var $filter = array();

    /**
     * A string that can be used as a search or filter statment ( eg MySQL 'WHERE' clause)
     * @var     array
     */
     var $filter_string;

    /**
     * An instance of a object for reading from the content source. Default is {@link MySqlReader}
     * @var     object
     */
     var $conn;

     /**
      * Accept the name of a content source and open a connection to it.
      * @param string $name name of the content source
      * @see openContentFetcher()
      * @return void
      */
    function NoAhContentReader( $name )
    {
         $this->NoAhContentSource($name);
         $this->loadSourceFile();
         // Open an connection to a content fetcher.
         $this->openContentFetcher();
    }
	
    /**
     * Creates an instance of {@link $fetch}
     * 
     * This method eventually choose between available 'reader classes'
     * The default being  NoAh MySQLReader.php  this class handles 
     * The physical fetching of content from the database. 
     * This would be the class file to mirror if we want to start fetcing
     * content from other datasources, then modify the following method
     * to support it.
     * 
     * @return void
     */
	function openContentFetcher()
	{
		require_once( CORE_PATH.'/class/noahlib/NoAhMySqlReader.php');
	    $this->conn = new NoAhMySqlReader( $this->dbconn );
	}
	
    /**
     * Returns a set of content records from our {@link $conn} Object.
     * @see setProperties()
     * @return array
     */
	function getContent()
	{
	    $this->passPropertiesForward();
	    return $this->conn->getContentRecords();
	}
	
    /**
     * Counts the total number of content records found in {@link $conn}
     * @see setProperties()
     * @return integer
     */
	function countContent()
	{
		$this->passPropertiesForward();
		return $this->conn->getContentCount();
	}

    /**
     * Counts the total number of content records found in {@link $conn}
     * @see setProperties()
     * @return integer
     */
	function paginateContent()
	{
		$this->passPropertiesForward();
        $page = array();
        $page['records'] = $this->conn->getContentCount();
        $page['page'] = $this->page;
        $page['last'] = ceil($page['records']/$this->limit);
        if ($page['last'] < 1 ) $page['last'] = 1;
        $page['next'] = ( $this->page == $page['last'] )? 0 : $this->page+1;
        $page['prev'] = ( $this->page == 1 ) ? 0 : $this->page-1;
        $page['from'] = $this->limit*($this->page-1)+1;
        $page['to'] = $this->limit+($page['from']-1);
        if ( $page['to'] > $page['records'] ) {
            $page['to'] = $page['records'];
        }
		return $page;
	}

    /**
     * Adds a filter to apply to our Content Source, sets up {@link $filter}
     * 
     * We search or filter the content returned to use from {@link getContent()}
     * if we first add filters to this class.
     * 
     * @param string $field name content piece/field to filter on
     * @param string $operand type of operand used in filter eg: =, >, <, !=
     * @param string $value that we want to filter against.
     * @return void
     */
	function addFilter( $field, $operand, $value )
	{   
        $n = count($this->filter);
	    $this->filter[$n]['field'] = $field;
	    $this->filter[$n]['operand'] = $operand;
	    $this->filter[$n]['value'] = $value;
	}

    /**
     * Allows use to add a full string to use as a filter, sets up {@link $filter_string}
     * 
     * Any value we set here will override {@link $filter}. For example if our content
     * source is a MySQL db we can use this method to supply a full ' WHERE ' clause
     * to use when content is retrieved.
     * 
     * @param string $filter_string string to use as a filter statement
     * @return void
     */
	function addFilterString( $filter_string )
	{
		$this->filter_string = $filter_string;
	}

    /**
     * Clears all values in {@link $filter} and {@link $filter_string}
     * @return void
     */
    function clearFilter()
    {
        $this->filter = array();
        $this->filter_string = '';
    }
	
    /**
     * Sets the values for {@link $sortby} and {@link $sortdir}
     * 
     * Before calling the {@link getContent()} method we can use this
     * method to set which field and which direction our content should
     * be sorted on.
     * 
     * @param string $fields The name of the content piece to sort on
     * @param string $direction the direction sorting should run 'ASC' or 'DESC'
     * @return void
     */
	function setSort( $fields, $direction )
	{
		$this->sortby = $fields;
		$this->sortdir = $direction;
	}

	// Function sets the limit, or number of records to return
	// when fetching content.
    /**
     * Sets the value of {@link $limit} and controls number of content records
     * returned with {@link getContent()}
     * @param integer $limit number of content records to return
     * @return void
     */
	function setLimit( $limit )
	{
		$this->limit = $limit;
	}
    /**
     * Sets the value of {@link $limit} and controls number of content records
     * returned with {@link getContent()}
     * @param integer $limit number of content records to return
     * @return void
     */
	function setPage( $page )
	{
		$this->page = $page;
	}
	
    /**
     * Sets {@link $columns} and returns the full array 
     * 
     * {@link $columns} contains name,label combination of the columns
     * that get returned with {@link getContent()} and can be handy when
     * wanting to display content in a table format.
     * 
     * @see setAllColumns()
     * @return array
     */
	function getColumns()
	{   
	    if ( ! $this->columns ) $this->setAllColumns();
	    
	    // Check if our primary key is already in the array of
	    // columns we are wanting to fetch
	    //if ( ! in_array($this->prime,$this->columns) ){
	    //    $columns[0]['label'] = $this->prime;
	    //    $columns[0]['name'] = $this->prime;
	    //    $n=1;
	    // } else {
 	    //    $n=0;
	    // }
        $columns = array();
        $n=0;
	    foreach( $this->columns as $i=>$column ) {
	       $pi = $this->pieceIndex($column);
	       $columns[$n]['label'] = $this->pieces[$pi]->label; 
	       $columns[$n]['name'] = $this->pieces[$pi]->name; 
	    $n++;
	    }
	   return $columns;
	}
	
    /**
     * Adds all content Pieces to {@link $columns}
     * 
     * If no columns are explicitly requested then this method sets
     * all content pieces in the current content source to be returned as columns.
     * 
     * @see getColumns()
     * @return void
     */
	function setAllColumns()
	{
	   // First, clear all our columns
	   $this->clearColumns();
	   foreach($this->getPieces() as $i=>$piece ) {
	      $this->columns[] = $piece->name;
	   }
	}
	
    /**
     * Adds a single column to {@link $columns}
     * @param string $column name of the column to add
     * @return void
     */
	function addColumn( $column )
	{
	   $this->columns[] = $column;
	}
	
    /**
     * Adds an array of columns to {@link $columns}
     * @param array $columns an array of columns to add
     * @return void
     */
	function addColumns( $columns )
	{
	   $this->columns = $columns;
	}
	
    /**
     * Clears all values from {@link $columns}
     * @return void
     */
	function clearColumns()
	{
		$this->columns = array();
	}
	
    /**
     * This method allow (good or bad) developers to pass thier
     * own custom properties to the {@link $fetch} object
     * 
     * @param string $name name of the custom property
     * @param string $value value of the custom property
     * @return void
     */
	function setProperty($name,$value)
	{
		$this->custom_prop[$name] = $value;
	}
	
    /**
     * Builds an array of properties found in this class.
     * @return array
     */
	function getContentProperties()
	{
	   $props = array();
       $props['sortby'] = $this->sortby;
       $props['sortdir'] = $this->sortdir;
       $props['page'] = $this->page;
       $props['limit'] = $this->limit;
       $props['filter'] = $this->filter; 
       $props['filter_string'] = $this->filter_string;
       $props['custom_prop'] = $this->custom_prop;
       $props['columns'] = $this->getColumns();
       $props['pieces'] = $this->pieces;
       $props['source'] = $this->sourceDetails();
       return $props;
	}
	
    /**
     * Passes an array of content properties to the {@link $fetch} object
     * @see getContentProperties()
     * @see openContentFetcher()
     * @return void
     */
	function passPropertiesForward()
	{
	    $this->conn->setProperties( $this->getContentProperties() );
	}

    /**
     * Specific to MySQLReader class this method will return SQL used
     * when the {@link getContent()} method is called.
     * @return string
     */
    function getSql(){
        return $this->conn->getSql();
    }


}


?>
