<?php
// $Id: NoAhContentSource.php,v 1.6 2005/03/22 01:57:22 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhContentSource.php provides methods for handling a NoAh defined Content Source
 *
 * NoAh provides the ability to capture and manage metadata about specific sources
 * of content. This class allows for the creation and use of a NoAh Content Source.
 *
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */
 
/**
 * NoAhContentSource provides methods for handling a NoAh defined Content Source
 * 
 * A NoAhContentSource decribes details on where content is stored and what content
 * pieces can be found. Properties for this class can be added usign the 
 * NoAhObject:setAttribute() method.
 * 
 * @package noahlib
 */
class NoAhContentSource extends NoAhObject
{
    /**
     * The RDF-XML file that defines this content source
     * @var     string
     */
    var $filepath;
    /**
     * The database connection to use for Database Content Sources
     * @var     string
     */
    var $dbconn;
    /**
     * The name of the database table for this content source
     * @var     string
     */
    var $table;
    /**
     * The name of the primary key for this content source
     * @var     string
     */
    var $prime;
    /**
     * An array of XML data read from {@link $filepath} 
     * @var     array
     */
    var $xml;
    /**
     * The label or user friendly {@link $name} of this content source
     * @var     string
     */
    var $label;
    /**
     * The RDF URI of this content source
     * @var     string
     */
    var $uri;
    /**
     * An array of {@link NoAhContentPiece}s contained in this content source
     * @var     array
     */
    var $pieces = array();
    /**
     * The name of this content source
     * @var     string
     */
    var $name;

    /**
     * The type of access granted to this content source
     * @var     integer
     */
    var $access;

    /**
     * A flag which can be set to lock changes from being made to this content source
     * @var     boolean
     */
    var $allowedit;
    /**
     * The defined type of this content source
     * @var     string
     */
    var $type;

    /**
     * A description of this content source
     * @var     string
     */
    var $desc;


   /**
    * Constructor :
    */
    /**
     * Class Constructor sets {@link $name} and {@link $filepath}
     * 
     * Accepts the name of our content source and also a flag which can
     * be set to 1/true if content source should be immediately loaded.
     * 
     * @param string $name name of our content source.
     * @param string $load a flag set to true if {@link loadSourceFile()} should be executed
     * @return void
     */
    function NoAhContentSource( $name, $load=1 )
    {
        // Set the Name of our content source.
        $this->name = $this->normalizeName($name);
        // Check to see if we can find a file for this source.
        if ( is_file(CORE_PATH.'/xml/'.$this->name ) ) {
            $this->filepath = CORE_PATH.'/xml/'.$this->name;
        }else if(is_file(XML_PATH.'/'.$this->name )){
            $this->filepath = XML_PATH.'/'.$this->name;
        }
        if ( $load ) {
        	$this->loadSourceFile();
        }
    }

    /**
     * Loads the content source XML file and passes contents to {@link readXML()}
     * @return boolean true if xml file was found and read, false if not.
     */
    function loadSourceFile()
    {
        if ( ! $this->filepath ) return false;
        if ( $this->xml ) return true;
        $nrap = new NoAhRap($this->filepath);
        $this->xml = $nrap->getData();
        $this->readXML();
        unset($nrap);
        return true;
    }
    

    /**
     * Converts string to valid XML file name
     * 
     * Adds extension .xml and prefix source_ to to convert 
     * supplied string into valid format of a content source xml file.
     * @param string $name  name to convert
     * @return string
     */
    function normalizeName($name)
    {
        if ( substr(strtolower($name),-4)!='.xml') {
            $name = $name.'.xml';
        }
        if ( strpos($name,'source_')===false ) {
            $name = 'source_'.$name;
        }
        return strtolower($name);
    }

    /**
     * Adds a content piece to {@link $pieces}
     * @param object $piece  a NoAhContentPiece object
     * @return void
     */
    function addPiece( &$piece )
    {
        $this->pieces[] = $piece;
    }
    /**
     * Adds an array of content pieces to source.
     * 
     */
    /**
     * Adds an array of pieces to {@link $pieces}
     * @param array $pieces array of NoAhContentPiece objects
     * @return void
     */
    function addPieces( $pieces )
    {
        foreach($pieces as $k=>$v ){
            $this->pieces[] = $v;
        }
    }

    /**
     * Returns an array of all {@link $pieces}
     * @return array
     */
    function getPieces(){
        usort($this->pieces, 'orderPieceComparison');
        return $this->pieces;
    }

    /**
     * Removes a single item from {@link $pieces} by name
     * 
     * NOTE: This method doesn't reindex {@link $pieces} after dropping one.
     * I am unsure at this time if that is necessary or not.
     * 
     * @param string $name the name of the piece to drop
     * @return void
     */
    function dropPiece($name)
    {
        foreach($this->pieces as $i=>$piece ){
            if ($piece->name == $name) {
                unset($this->pieces[$i]);
            }
        }
    }

    /**
     * Checks {@link $pieces} for the exsistence of specific name
     * 
     * Returns true if piece found and false if not.
     * 
     * @param string $name name of the content piece to look for.
     * @return boolean
     */
    function hasPiece($name)
    {
        $has = false;
        foreach($this->pieces as $i=>$piece ){
            if ($piece->name == $name) {
                $has=true;
            }
        }
        return $has;
    }

    /**
     * Looks over the results of {@link loadSourceFile()} and adds
     * properties to this class and pieces to {@link $pieces}
     * @see loadSourceFile()
     * @see addPiece()
     * @return void
     */
    function readXML()
    {
        // Pull out individual arrays showing 
        foreach($this->xml as $uri=>$data){
            // look for source namespace.
            if ( substr($uri,0,7)=='source/') {
                $this->xsource = $data;
            }
            if ( substr($uri,0,7)=='pieces/') {
                $this->xpieces = $data;
            }
        }
        require_once(CORE_PATH.'/class/noahlib/NoAhContentPiece.php');
        
        // loop over each xpieces and add to pieces.
        foreach($this->xpieces as $i=>$uri){
            if (isset($this->xml[$uri]) && $this->xml[$uri] ) {
              $myPiece = new NoAhContentPiece();
              // Loop over piece details and add attributes to $myPiece
              foreach( $this->xml[$uri] as $n=>$v ){
                 $myPiece->setAttribute($n,$v);
                 if ($n=='join'){ // If the piece has a noah:join value
                   // Loop over each join and grab join uri
                   $myjoins = array();
                   foreach($this->xml[$v] as $ji=>$juri){
                   	 if ($juri == 'Seq') continue; 
                   	 $myjoins[] = $this->xml[$juri];
                   }
                   $myPiece->setAttribute('joins',$myjoins);
                 }
              }
              $this->addPiece($myPiece);
              unset($myPiece);
            }
        }

        // loop over source details and grab somedetails
        foreach($this->xsource as $i=>$uri)
        {
            $this->$i = $uri;
        }
    }

    /**
     * Returns and index of a single {@link $pieces} based on its name
     * @param string $name name of the piece to fetch index for
     * @return integer
     */
    function pieceIndex($name)
    {
        $id = 0;
        foreach($this->pieces as $i=>$piece ){
            if ($piece->name == $name) {
                $id=$i;
            }
        }
        return $id;
    }

    /**
     * Returns an array of properties for this content source
     * @return array
     */
    function sourceDetails()
    {
       foreach($this as $k=>$v)
       { 
         if ($k=='pieces'||$k=='xml') continue;
         $det[$k]=$v;
       }
     return $det;
    }
}

if ( ! function_exists('orderPieceComparison') ) {
    function orderPieceComparison($a,$b){
      if ($a->order == $b->order) {
           return 0;
       }
       return ($a->order < $b->order) ? -1 : 1;
    }
}


?>
