<?php
// $Id: NoAhFormBuilder.php,v 1.1 2005/03/22 01:59:55 kerkness Exp $
   
//*************************************************************************//
// This file is part of NoAh.
// 
// NoAh is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// NoAh is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with NoAh; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//*************************************************************************//

/**
 * NoAhFormBuilder.php provides methods for generating forms and form elements
 *
 * For all content pieces managed in NoAh, we keep track of what the most optimal
 * input type is best used to edit that content piece. This class helps turn that
 * information into actual form elements that can be used in a form.
 *
 * @author Ryan Mayberry < phparch at gmail dot com >
 * @package noahlib
 */
 
/**
 * NoAhFormBuilder provides methods for generating forms and form elements
 * 
 * Properties for this class can be added using the NoAhObject::setAttribute()
 * method.
 * 
 * @package noahlib
 */
class NoAhFormBuilder extends NoAhObject
{

    /**
     * @var     array
     */
    var $fields = array();

    /**
     * @var     array
     */
    var $elements = array();

    /**
     * @var     string
     */
     var $extra;

    /**
     * @var 	string
     */
     var $name;

    /**
     * @var 	string
     */
     var $action;

    /**
     * @var 	string
     */
     var $method;

    /**
     * @var 	string
     */
     var $format;
     

     /**
      * Class constructor sets {@link $name} and {@link $format}
      * @param string $name name of our form
      * @param string $format the format to render form elements 'html' or 'xul'
      * @return void
      */
    function NoAhFormBuilder( $name, $format='html' )
    {
         $this->name = $name;
         $this->format = $format;
    }

    /**
     * Adds a single form field to {@link $fields}
     * 
     * These details can be found in a NoAhContentPiece
     * 
     * @param string $name Name of our form field
     * @param string $label Label of our field
     * @param string $input Type of input field to render
     * @param string $value Current/Default value of our input field
     * @return void
     */
	function addFormField( $name, $label, $input, $value )
	{
		$this->fields[$name]['label'] = $label;
		$this->fields[$name]['input'] = $input;
		$this->fields[$name]['value'] = $value;
	}
	
    /**
     * Adds an array of content piece objects to {@link $fields}
     * @see addFormField()
     * @param array $pieces an array of NoAhContentPieces
     * @return void
     */
	function addContentPieces( $pieces )
	{
		foreach($pieces as $i=>$piece )
		{
			$this->addFormField( $piece->name, $piece->label, $piece->input, $piece->default );
			if ( $piece->joins ) {
			  $this->setProperty( $piece->name, 'options', $piece->getOptionsFromJoin());
			}  
		}
	}
	
    /**
     * Adds an array of field values to {@link $fields}
     * @see setValue()
     * @param array $values  and array of name/value pairs
     * @return void
     */
    function setValueArray( $values ) 
    {
    	foreach( $values as $name => $value )
    	{
    		$this->setValue($name, $value);
    	}
    }
    
    /**
     * Updates or sets the value of a single value in {@link $fields}
     * @param string $name of the field to set value for
     * @param string $value to set field to
     * @return void
     */
    function setValue( $name, $value )
    {
    	$this->fields[$name]['value'] = $value;
    }

    /**
     * Sets a property name/value pair to a single {@link $fields}
     * @param string $name name of field to add property to
     * @param string $prop name of the property to add
     * @param string $value value of the property
     * @return void
     */
    function setProperty( $name, $prop, $value )
    {
    	$this->fields[$name]['prop'][$prop] = $value;
    }
    
    /**
     * Renders an array of rendered input tags for all {@link $fields}
     * @see renderField()
     * @return array
     */
	function renderFields()
	{
		$fields = array();
		foreach($this->fields as $name => $field )
		{
			$fields[$name] = $this->renderField($name);
		}
		return $fields;
	}
	
    /**
     * Renders the input tag and form element details for a single {@link $fields}
     * @param string $name name of the field to render
     * @return array
     */
	function renderField( $name )
	{
		$field['label'] = ( isset($this->fields[$name]['label']) )? $this->fields[$name]['label'] : $name;
		$field['body'] = $this->renderFieldBody( $name );
		$field['input'] = $this->fields[$name]['input'];
        $field['helptip'] = (isset($this->fields[$name]['prop']['helptip']))? $this->fields[$name]['prop']['helptip'] : '' ;
	  return $field;	
	}
	
	/**
	 * Renders all form elements including form html headers for html forms
	 * @return array
	 */
	function renderForm()
	{
	   $form = array();
	   $form['header'] = '<form id="'.$this->name.'" action="'.$this->action.'" method="'.$this->method.'" '.$this->extra.'>';
	   $form['fields'] = $this->renderFields();
	   $form['footer'] = '</form>';	
	   return $form;
	}

    /**
     * Renders the just the input tag od a specific {@link $fields}
     * @param string $name of the field to render
     * @return string
     */
    function renderFieldBody( $name )
    {
    $body = '';
    $field = $this->fields[$name];
    $input_type = ( isset($field['input']) )? $field['input'] : 'html' ;
    $field_label = ( isset($field['input']) )? $field['label'] : $name ;
    $field_value = ( isset($field['value']) )? $field['value'] : '' ;
    $field_extra = ( isset($field['prop']['extra']) )? $field['prop']['extra'] : ''; 
    $field_class = ( isset($field['prop']['class']) )? $field['prop']['class'] : ''; 
    //  Create a Form Object of either xoopsforms or xulforms
    // depending on current format
    switch($this->format){
    // *********************** START HTML FORM ELEMENTS 
    case 'html':
    default:
    	require_once(CORE_PATH.'/class/xoopsform/xoopsformloader.php');
        			
    
    	switch ( $input_type ){
        case 'text':
        default:
    	$size = (isset($field['prop']['size']))?$field['prop']['size']:'30';
    	$maxlength = (isset($field['prop']['maxlength']))?$field['prop']['maxlength']:'255';
    	$html = new XoopsFormText($field_label, $name, $size, $maxlength, $field_value);
    	break;
        case 'password':
    	$size = (isset($field['prop']['size']))?$field['prop']['size']:'30';
    	$maxlength = (isset($field['prop']['maxlength']))?$field['prop']['maxlength']:'255';
    	$html = new XoopsFormPassword($field_label, $name, $size, $maxlength, $field_value);
    	break;
    	case 'textarea':
    	$rows = (isset($field['prop']['rows']))?$field['prop']['rows']:'5';
    	$cols = (isset($field['prop']['cols']))?$field['prop']['cols']:'50';
    	$html = new XoopsFormTextArea($field_label, $name, $field_value, $rows, $cols);
    	break;
    	case 'select':
    	$size = (isset($field['prop']['size']))?$field['prop']['size']:1;
    	$multiple = (isset($field['prop']['multiple']))?$field['prop']['multiple']:false;
    	$options = (isset($field['prop']['options']))?$field['prop']['options']:array(''=>'-');
    	$html =  new XoopsFormSelect($field_label, $name, $field_value, $size=1, $multiple=false);		
    	$html->addOptionArray($options);
    	break;
    	case 'checkbox':
    	$options = (isset($field['prop']['options']))?$field['prop']['options']:array(''=>'-');
    	$html = new XoopsFormCheckBox($field_label, $name, $field_value);
    	$html->addOptionArray($options);
    	break;
    	case 'hidden':
    	$html = new XoopsFormHidden($name, $field_value);
    	break;	
        case 'datetime':
        $defdate = ( $field_value )? $field_value : mktime(0,0,0,date('m'),date('d'),date('Y'));        
    	$size = (isset($field['prop']['size']))?$field['prop']['size']:15;
        $html = new XoopsFormDateTime($field_label, $name, $size, $defdate );
        break;
        case 'imgupload':
    	$maxfilesize = (isset($field['prop']['maxfilesize']))?$field['prop']['maxfilesize']:99999999;
        $html = new XoopsFormFile($field_label, $name, $maxfilesize);
        break;
    	case 'button':
    	$type = (isset($field['prop']['type']))?$field['prop']['type']:'button';
    	$html = new XoopsFormButton($field_label, $name, $field_value, $type);
    	break;	
    	case 'submit':
    	$html = new XoopsFormButton($field_label, $name, $field_value, 'submit');
    	break;	
                   
    	}
        // Set EXTRA Value if present.
        if ( $field_extra ) {
            $html->setExtra($field_extra);
        }
        // Set CLASS Value if present.
    	
    	if ( is_object($html) ){
    	   $body = $html->render();
    	}
    						
    break;
    // *********************** END HTML FORM ELEMENTS 
    // *********************** START XUL FORM ELEMENTS 
    case 'xul':
    	require_once(CORE_PATH.'/class/xulform/xulformloader.php');
    break;
    // *********************** END XUL FORM ELEMENTS 
    }
		
	  return $body;	
	}

}


?>
